#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#
#     Authors:
#       Joan Sala Calero (joan.sala@bsc.es)

from pathlib import Path
import sys

# Custom packages
from dqc_downloader.utils_download import *

def main():

    # Params
    if len(sys.argv) != 3:
        print('USAGE: python download_reanalysis.py <config_file> <credentials_file>')
        exit()
    else:
        conf_path = sys.argv[1]
        cred_path = sys.argv[2]

    # General configuration
    with open('./conf/download_options.json') as json_file:
        conf = json.load(json_file)
        outputdir = conf['outdir']
        logdir = conf['logdir']
        cds_forms_c3s_location = Path(conf['formsdir'])

    # URL and key to download
    with open(cred_path) as json_file:
        cred = json.load(json_file)

    # Read list of products to download
    with open(conf_path) as json_file:
        data = json.load(json_file)

        # For every dataset
        for prod in data:

            # Get constraints for this dataset
            constraints = get_cds_forms_json(prod['dataset'], cds_forms_c3s_location, "constraints")
            fformat = prod['form']['format']

            # For every variable
            for v in prod['variables']:

                # Output directory [one dataset per directory]
                outdir = os.path.join(outputdir, '{}/{}/{}'.format(prod['group'], prod['dataset'], v))
                print("---------------------------------------")
                print("Directory = {}".format(outdir))
                if not os.path.exists(outdir):
                    os.makedirs(outdir)

                # Count downloads
                down = 0
                for c in constraints:

                    # Filter per variable
                    if v in c['variable']:

                        # Decide product type to download
                        if 'product_type' in prod:
                            pts = prod['product_type']
                        else:
                            pts = c['product_type']

                        # Download per [dataset/year/month/day/variable/product_type] - all times
                        for y in c['year']:
                            for m in c['month']:
                                for p in pts:

                                    # Group by month
                                    d = '01'
                                    outfile = "{dir}/{prod}-{year}{month}{day}.{format}".format(
                                        dir=outdir, var=v, year=y, month='{:02d}'.format(int(m)),
                                        day='{:02d}'.format(int(d)), format=fformat, prod=p
                                    )

                                    # Download per month
                                    call_download(cred, logdir, outfile, fformat, c, y, m, d, v, prod, groupby='monthly')
                                    down+=1
                # Files count
                print("Files = {}".format(down))


if __name__ == "__main__":
    main()