#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#
#     Authors:
#       Joan Sala Calero (joan.sala@bsc.es)

import os
import json

# Custom packages
from C3S512.cds import downloader

"""
Downloader, grouping files
"""
def call_download(cred, logdir, outfile, fformat, c, y, m, d, v, p, groupby='monthly'):

    # Build form per file [COPY]
    form = c.copy()
    form['variable'] = v
    if groupby == 'daily':
        form['year'] = y
        form['month'] = m
        form['day'] = d
    elif groupby == 'monthly':
        form['year'] = y
        form['month'] = m
        form['day'] = ['{:02d}'.format(dd) for dd in range(1, 32)]
    elif groupby == 'yearly':
        form['year'] = y
    else:
        raise Exception('Groupby option must be one of these (daily, monthly, yearly)')

    # Weird constrains.json behaviour [avoid MARS errors]
    if d != '0' and m != '0' and d != '00' and m != '00':

        # Download or skip
        if not os.path.exists(outfile):
            print("DOWN = {}".format(os.path.basename(outfile)))
            downloader(p['dataset'], form, outfile, logpath=logdir, key=cred['key'], user=cred['url'])
        else:
            print("SKIP = {}".format(os.path.basename(outfile)))

"""
Return full constraints/mapping object
"""
def get_constraints(dataset_name, cds_forms_c3s_location, json_name):

    try:
        json_path = cds_forms_c3s_location.joinpath("cds-forms-c3s", dataset_name, json_name + ".json")
        with open(json_path, "r") as cj:
            json_data = json.load(cj)
    except:
        print('Could not open: {}'.json_path)

    return json_data

"""
Return filtered constraints/mapping object
"""
def get_filtered_constraints(dataset_name, cds_forms_c3s_location, json_name, var, origin, system):

    res = []
    try:
        json_path = cds_forms_c3s_location.joinpath("cds-forms-c3s", dataset_name, json_name + ".json")
        with open(json_path, "r") as cj:
            json_data = json.load(cj)

        # Filter out variable
        for c in json_data:
            if var in c['variable'] and origin in c['originating_centre'] and system in c['system']:
                c['variable'] = var
                res.append(c)

    except:
        print('Could not open: {}'.json_path)

    return res

"""
Get row (month, year)
"""
def get_row_dates(row):
    years = row['year']
    months = row['month']
    if isinstance(years, str):
        years = [years]
    if isinstance(months, str):
        months = [months]

    return years, months